"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgConnectionArgFilterCompositeTypeAttributesPlugin = void 0;
const { version } = require("../package.json");
exports.PgConnectionArgFilterCompositeTypeAttributesPlugin = {
    name: "PgConnectionArgFilterCompositeTypeAttributesPlugin",
    version,
    schema: {
        hooks: {
            GraphQLInputObjectType_fields(inFields, build, context) {
                let fields = inFields;
                const { extend, sql, inflection, graphql: { isNamedType }, options: { connectionFilterAllowedFieldTypes }, } = build;
                const { fieldWithHooks, scope: { pgCodec: rawCodec, isPgConnectionFilter }, Self, } = context;
                if (!isPgConnectionFilter ||
                    !rawCodec ||
                    !rawCodec.attributes ||
                    rawCodec.isAnonymous) {
                    return fields;
                }
                const codec = rawCodec;
                for (const [attributeName, attribute] of Object.entries(codec.attributes)) {
                    const behavior = build.pgGetBehavior([
                        attribute.codec.extensions,
                        attribute.extensions,
                    ]);
                    if (!build.behavior.matches(behavior, "filter", "filter")) {
                        continue;
                    }
                    // keep only the composite type attributes
                    if (!attribute.codec.attributes) {
                        continue;
                    }
                    const fieldName = inflection.attribute({
                        codec,
                        attributeName,
                    });
                    const NodeType = build.getGraphQLTypeByPgCodec(attribute.codec, "output");
                    if (!NodeType || !isNamedType(NodeType)) {
                        continue;
                    }
                    const nodeTypeName = NodeType.name;
                    // Respect `connectionFilterAllowedFieldTypes` config option
                    if (connectionFilterAllowedFieldTypes &&
                        !connectionFilterAllowedFieldTypes.includes(nodeTypeName)) {
                        continue;
                    }
                    const filterTypeName = inflection.filterType(nodeTypeName);
                    const CompositeFilterType = build.getTypeByName(filterTypeName);
                    if (!CompositeFilterType) {
                        continue;
                    }
                    fields = extend(fields, {
                        [fieldName]: fieldWithHooks({
                            fieldName,
                            isPgConnectionFilterField: true,
                        }, () => ({
                            description: `Filter by the object’s \`${fieldName}\` field.`,
                            type: CompositeFilterType,
                            // TODO: applyPlan
                        })),
                    }, "");
                }
                return fields;
            },
        },
    },
};
//# sourceMappingURL=PgConnectionArgFilterCompositeTypeAttributesPlugin.js.map