"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PgConnectionArgFilterComputedAttributesPlugin = void 0;
const pg_1 = require("@dataplan/pg");
const utils_1 = require("./utils");
const { version } = require("../package.json");
exports.PgConnectionArgFilterComputedAttributesPlugin = {
    name: "PgConnectionArgFilterComputedAttributesPlugin",
    version,
    schema: {
        hooks: {
            GraphQLInputObjectType_fields(inFields, build, context) {
                let fields = inFields;
                const { extend, sql, inflection, connectionFilterOperatorsDigest, options: { connectionFilterComputedColumns }, } = build;
                const { fieldWithHooks, scope: { pgCodec: codec, isPgConnectionFilter }, Self, } = context;
                if (!isPgConnectionFilter ||
                    !codec ||
                    !codec.attributes ||
                    codec.isAnonymous) {
                    return fields;
                }
                // TODO: This may need to change once V5 fixes the need for it
                const source = Object.values(build.input.pgRegistry.pgResources).find((s) => s.codec === codec && !s.parameters && !s.isUnique);
                if (!source) {
                    return fields;
                }
                const computedAttributeResources = (0, utils_1.getComputedAttributeResources)(build, source);
                for (const computedAttributeResource of computedAttributeResources) {
                    // Must return a scalar or an array
                    if (!computedAttributeResource.isUnique) {
                        continue;
                    }
                    if (computedAttributeResource.codec.attributes) {
                        continue;
                    }
                    if (computedAttributeResource.codec === pg_1.TYPES.void) {
                        continue;
                    }
                    const digest = connectionFilterOperatorsDigest(computedAttributeResource.codec);
                    if (!digest) {
                        continue;
                    }
                    const OperatorsType = build.getTypeByName(digest.operatorsTypeName);
                    if (!OperatorsType) {
                        continue;
                    }
                    const behavior = build.pgGetBehavior([
                        computedAttributeResource.extensions,
                    ]);
                    const defaultBehavior = connectionFilterComputedColumns
                        ? "filterBy"
                        : "";
                    if (!build.behavior.matches(behavior, "filterBy", defaultBehavior)) {
                        continue;
                    }
                    const { argDetails, makeFieldArgs, makeArgs, makeExpression } = build.pgGetArgDetailsFromParameters(computedAttributeResource, computedAttributeResource.parameters.slice(1));
                    // Must have only one required argument
                    if (argDetails.some((a) => a.required)) {
                        continue;
                    }
                    // Looks good
                    const fieldName = inflection.computedAttributeField({
                        resource: computedAttributeResource,
                    });
                    const functionResultCodec = computedAttributeResource.codec;
                    fields = build.extend(fields, {
                        [fieldName]: fieldWithHooks({
                            fieldName,
                            isPgConnectionFilterField: true,
                        }, {
                            description: `Filter by the object’s \`${fieldName}\` field.`,
                            type: OperatorsType,
                            applyPlan($where, fieldArgs) {
                                if (typeof computedAttributeResource.from !== "function") {
                                    throw new Error(`Unexpected...`);
                                }
                                const expression = computedAttributeResource.from({
                                    placeholder: $where.alias,
                                });
                                const $col = new pg_1.PgConditionStep($where);
                                $col.extensions.pgFilterAttribute = {
                                    codec: functionResultCodec,
                                    expression,
                                };
                                fieldArgs.apply($col);
                            },
                        }),
                    }, "");
                }
                return fields;
            },
        },
    },
};
//# sourceMappingURL=PgConnectionArgFilterComputedAttributesPlugin.js.map